@@ -1,11 +1,12 @@
+import collections
 import copy
 import typing as tp
-import collections
+
+from satella.coding.typing import T
 
 ITER_KEYS = 0
 ITER_VALUES = 1
 ITER_ITEMS = 2
-T = tp.TypeVar('T')
 
 
 class DictDeleter:
@@ -112,14 +113,14 @@ class ListDeleter(tp.Generic[T]):
 
     You can pass any type of object here, as long as it supports pop(position) and __getitem__
     """
-    __slots__ = ('list_to_process', 'current_index', 'indices_to_delete', 'direction')
+    __slots__ = 'list_to_process', 'current_index', 'indices_to_delete', 'direction'
 
     def __init__(self, list_to_process: tp.MutableSequence[T]):
         self.list_to_process = list_to_process
         self.direction = DIR_FORWARD
         # pointer to currently processed element
         self.current_index = -1 if self.direction == DIR_FORWARD else len(self.list_to_process)
-        self.indices_to_delete = set()      # type: tp.Set[int]
+        self.indices_to_delete = set()  # type: tp.Set[int]
 
     def __enter__(self) -> 'ListDeleter':
         return self
@@ -139,7 +140,7 @@ class ListDeleter(tp.Generic[T]):
         self.indices_to_delete = set()
         return self
 
-    def __next__(self):
+    def __next__(self) -> T:
         if self.direction == DIR_BACKWARD and self.current_index == 0:
             raise StopIteration('First element reached')
         if self.direction == DIR_FORWARD and self.current_index == len(self.list_to_process) - 1:
@@ -152,8 +153,7 @@ class ListDeleter(tp.Generic[T]):
         :return: the next element
         :raises StopIteration: no more entries
         """
-        next(self)
-        return self.list_to_process[self.current_index]
+        return next(self)
 
     def prev(self) -> T:
         """
