@@ -1,12 +1,13 @@
-import typing as tp
 import operator
+import typing as tp
 
-__all__ = ['x']
+from satella.coding.typing import Predicate
+from satella.configuration.schema import Descriptor
+
+__all__ = ['x', 'build_structure', 'PredicateClass']
 
 import warnings
 
-PredicateType = tp.Callable[[tp.Any], bool]
-
 
 def _nop(v: tp.Any) -> tp.Any:
     return v
@@ -14,26 +15,30 @@ def _nop(v: tp.Any) -> tp.Any:
 
 def make_operation_two_args(operation_two_args: tp.Callable[[tp.Any, tp.Any], tp.Any],
                             docstring: tp.Optional[str] = None,
-                            swap_order: bool = False) -> PredicateType:
-    def operation(self, a) -> PredicateType:
-        if isinstance(a, Predicate):
+                            swap_order: bool = False) -> Predicate:
+    def operation(self, a: tp.Callable) -> Predicate:
+        if isinstance(a, PredicateClass):
             if swap_order:
                 def op(v):
                     return operation_two_args(a(v), self(v))
-                return Predicate(op)
+
+                return PredicateClass(op)
             else:
                 def op(v):
                     return operation_two_args(self(v), a(v))
-                return Predicate(op)
+
+                return PredicateClass(op)
         else:
             if swap_order:
                 def op(v):
                     return operation_two_args(a, self(v))
-                return Predicate(op)
+
+                return PredicateClass(op)
             else:
                 def op(v):
                     return operation_two_args(self(v), a)
-                return Predicate(op)
+
+                return PredicateClass(op)
 
     if docstring:
         operation.__doc__ = docstring
@@ -42,11 +47,12 @@ def make_operation_two_args(operation_two_args: tp.Callable[[tp.Any, tp.Any], tp
 
 
 def make_operation_single_arg(operation: tp.Callable[[tp.Any], tp.Any],
-                              docstring: tp.Optional[str] = None) -> PredicateType:
-    def operation_v(self) -> PredicateType:
+                              docstring: tp.Optional[str] = None) -> Predicate:
+    def operation_v(self) -> Predicate:
         def operate(v):
             return operation(self.operation(v))
-        return Predicate(operate)
+
+        return PredicateClass(operate)
 
     if docstring:
         operation_v.__doc__ = docstring
@@ -65,14 +71,14 @@ def _one_of(a, values: tp.Container) -> bool:
     return a in values
 
 
-class Predicate:
+class PredicateClass:
     """
     A shorthand to create lambdas using such statements, for example:
 
     >>> add_two = x + 2
     >>> assert add_two(2) == 4
     """
-    __slots__ = ('operation', )
+    __slots__ = ('operation',)
 
     def __init__(self, operation: tp.Callable[[tp.Any], tp.Any] = _nop):
         self.operation = operation
@@ -80,36 +86,83 @@ class Predicate:
     str = make_operation_single_arg(str, 'Call str() on predicate')
     int = make_operation_single_arg(int, 'Call int() on predicate')
     float = make_operation_single_arg(float, 'Call float() on predicate')
+    false = make_operation_single_arg(lambda y: not y, 'Return a predicate checking whether '
+                                                       'value is False')
 
     def __call__(self, *args) -> bool:
         if len(args) == 0:
-            return Predicate(lambda y: self.operation(y)())
+            return PredicateClass(lambda y: self.operation(y)())
         else:
             return self.operation(args[0])
 
-    def has_keys(self, *keys) -> PredicateType:
+    def has_keys(self, *keys) -> Predicate:
         """
         Return a predicate checking whether this value has provided keys
         """
         return make_operation_two_args(_has_keys)(self, keys)
 
-    def one_of(self, *values) -> PredicateType:
+    def one_of(self, *values) -> Predicate:
         """
         Return a predicate checking if x is amongst values
         """
         return make_operation_two_args(_one_of)(self, values)
 
-    def has_p(self, predicate: 'Predicate') -> PredicateType:
+    def has_p(self, predicate: 'PredicateClass') -> Predicate:
         """
         An old name for has().
 
         It's deprecated. Use has() instead
+
+        .. deprecated:: 2.14.22
         """
         warnings.warn('This is deprecated and will be removed in Satella 3.x.'
                       'Please use has() instead', DeprecationWarning)
         return self.has(predicate)
 
-    def has(self, predicate: 'Predicate') -> PredicateType:
+    def type(self) -> tp.Type:
+        """
+        Return a predicate returning the type of it's argument
+        """
+
+        def type_of(v):
+            return type(self.operation(v))
+
+        return PredicateClass(type_of)
+
+    def is_instance(self, *args):
+        """
+        Check if given value is one of instances.
+
+        :param args: will be passed as argument to isinstance
+        """
+
+        def is_instance(v):
+            return isinstance(self.operation(v), args)
+
+        return PredicateClass(is_instance)
+
+    def identity(self) -> Predicate:
+        """
+        Spawn another object with the same operation, but different identity.
+
+        Used for constructing dicts keyed by predicates.
+        """
+        return PredicateClass(self.operation)
+
+    def is_valid_schema(self, schema: tp.Optional[tp.Union[Descriptor, tp.Dict]] = None, **kwargs):
+        """
+        Check if given value has the correct schema.
+        The schema is the same as in
+        :py:meth:`~satella.coding.structures.DictObject.is_valid_schema`
+        """
+
+        def is_schema_correct(v):
+            from satella.coding.structures import DictObject
+            return DictObject(self.operation(v)).is_valid_schema(schema, **kwargs)
+
+        return PredicateClass(is_schema_correct)
+
+    def has(self, predicate: 'PredicateClass') -> Predicate:
         """
         Check if any element of the current value (which must be an iterable)
         returns True when applied to predicate
@@ -117,12 +170,14 @@ class Predicate:
         :param predicate: predicate that has to return True for at least one of this predicate's
             values
         """
+
         def op(v):
             for e in self.operation(v):
                 if predicate(e):
                     return True
             return False
-        return Predicate(op)
+
+        return PredicateClass(op)
 
     inside = make_operation_two_args(operator.contains,
                                      'Return a predicate checking if x is inside value')
@@ -172,5 +227,56 @@ class Predicate:
     __invert__ = make_operation_single_arg(operator.invert)
     __abs__ = make_operation_single_arg(abs)
 
+    def __hash__(self) -> int:
+        return hash(id(self))
 
-x = Predicate()
+
+x = PredicateClass()
+
+
+def build_structure(struct: tp.Union[tuple, list, dict],
+                    argument,
+                    final_operator=lambda y: y,
+                    nested_call=False) -> tp.Union[tuple, list, dict]:
+    """
+    Given a structure (tuple, list, dict) that contains x's as some of the elements,
+    build such a structure corresponding to given that all x's are replaced
+    by result of their calculation on argument.
+
+    Just note that if you're constructing dictionaries, use the .identity() method of predicate,
+    to randomize it's identity.
+
+    :param struct: structure to build
+    :param argument: argument
+    :param final_operator: an operator to call on the result
+    :param nested_call: internal, don't use
+    :return: analogous structure
+    """
+    if not nested_call:
+        v = build_structure(struct, argument, None, True)
+        return final_operator(v)
+
+    from satella.coding.structures import HashableWrapper
+    if isinstance(struct, tp.MutableMapping):
+        new_dict = {}
+        for key, value in struct.items():
+            key = build_structure(key, argument, None, True)
+            value = build_structure(value, argument, None, True)
+            new_dict[key] = value
+            return new_dict
+    elif isinstance(struct, tp.Sequence):
+        new_seq = []
+        for elem in struct:
+            if isinstance(elem, PredicateClass):
+                elem = elem(argument)
+            else:
+                elem = build_structure(elem, argument, None, True)
+            new_seq.append(elem)
+        return struct.__class__(new_seq)
+    elif isinstance(struct, HashableWrapper):
+        obj = getattr(struct, '_Proxy__obj')
+        return build_structure(obj, argument, None, True)
+    elif isinstance(struct, PredicateClass):
+        return struct(argument)
+    else:
+        return struct
