@@ -1,8 +1,9 @@
 from .enums import ComparableEnum, ComparableIntEnum, HashableIntEnum
 from .hashable import ComparableAndHashableBy, ComparableAndHashableByInt, \
-    OmniHashableMixin, HashableMixin
+    OmniHashableMixin, HashableMixin, ComparableAndHashableByStr
 from .strings import ReprableMixin, StrEqHashableMixin
 
 __all__ = ['ComparableIntEnum', 'ComparableEnum', 'ComparableAndHashableBy',
            'HashableIntEnum', 'ComparableAndHashableByInt', 'OmniHashableMixin',
-           'ReprableMixin', 'StrEqHashableMixin', 'HashableMixin']
+           'ReprableMixin', 'StrEqHashableMixin', 'HashableMixin',
+           'ComparableAndHashableByStr']
