@@ -2,10 +2,10 @@ import logging
 import math
 import typing as tp
 
-from satella.coding.decorators import wraps
+from satella.coding.decorators.decorators import wraps
 from satella.coding.recast_exceptions import rethrow_as
+from satella.coding.typing import T
 
-T = tp.TypeVar('T')
 logger = logging.getLogger(__name__)
 
 _SETTABLE_KEYS = {'_Proxy__obj', '_Proxy__wrap_operations'}
@@ -56,7 +56,6 @@ class Proxy(tp.Generic[T]):
     :param wrap_operations: whether results of operations returning something else should be
         also proxied.
     """
-    __slots__ = ('__obj', '__wrap_operations')
 
     def __init__(self, object_to_wrap: T, wrap_operations: bool = False):
         self.__obj = object_to_wrap  # type: T
