@@ -1,16 +1,35 @@
 import collections
+import enum
 import random
 import typing as tp
 
 from satella.coding.decorators import for_argument
-
+from .jsonify import jsonify
 from .merger import merge_series
+from .merge_list import merge_list
+from .percentile import percentile
+from .base64 import b64encode
+from .interpol import linear_interpolate
+from .words import hashables_to_int
 
-__all__ = ['stringify', 'split_shuffle_and_join', 'one_tuple',
-           'merge_series', 'pad_to_multiple_of_length', 'clip']
+__all__ = ['stringify', 'split_shuffle_and_join', 'one_tuple', 'none_if_false',
+           'merge_series', 'pad_to_multiple_of_length', 'clip', 'hashables_to_int',
+           'jsonify', 'intify', 'percentile', 'b64encode', 'linear_interpolate',
+           'merge_list']
 
-T = tp.TypeVar('T')
-Number = tp.Union[int, float]
+from satella.coding.typing import T, NoArgCallable, Appendable, Number, Predicate
+
+
+def none_if_false(y: tp.Any) -> tp.Optional[tp.Any]:
+    """
+    Return None if y is false, else return y
+
+    :param y: value to check
+    :return: None if y is false, else y
+    """
+    if not y:
+        return None
+    return y
 
 
 def clip(v: Number, minimum: Number, maximum: Number) -> Number:
@@ -30,10 +49,10 @@ def clip(v: Number, minimum: Number, maximum: Number) -> Number:
 
 
 @for_argument(list)
-def pad_to_multiple_of_length(seq: tp.Sequence[T], multiple_of: int,
+def pad_to_multiple_of_length(seq: Appendable[T], multiple_of: int,
                               pad_with: tp.Optional[T] = None,
-                              pad_with_factory: tp.Optional[tp.Callable[[], T]] = None) -> \
-        tp.List[T]:
+                              pad_with_factory: tp.Optional[NoArgCallable[T]] = None) -> \
+        Appendable[T]:
     """
     Make sequence multiple of length, ie. append elements to the sequence
     until it's length is a multiple of multiple_of.
@@ -56,32 +75,57 @@ def pad_to_multiple_of_length(seq: tp.Sequence[T], multiple_of: int,
     return seq
 
 
-def _stringify_none(str_none, stringifier):
+def _stringify_none(str_none: tp.Optional[str],
+                    stringifier: tp.Callable[[tp.Any], str]) -> tp.Optional[str]:
     if str_none:
         return stringifier(None)
     return None
 
 
-T = tp.TypeVar('T')
-
-
 def one_tuple(x: tp.Iterable[T]) -> tp.Iterator[tp.Tuple[T]]:
     """
     Change a sequence of iterables into a sequence that displays each element as
     a part of one-element tuple. Essentially syntactic sugar for:
 
-    >>> for y in x:
-    >>>     yield y,
+    >>> for z in x:
+    >>>     yield z,
 
     :param x: sequence to tupleify
     :return: a iterator of one-element tuples
     """
-    for y in x:
-        yield y,
+    for z in x:
+        yield z,
+
+
+def intify(v: tp.Any) -> int:
+    """
+    Attempt to convert v to an int.
+
+    None will be converted to 0.
+
+    Any object will have int() called on it.
+
+    Failing that, it's length will be taken.
+
+    Failing that, ValueError will be raised
+
+    :param v: parameter
+    :return: int representation
+    """
+    if v is None:
+        return 0
+
+    try:
+        return int(v)
+    except (TypeError, ValueError):
+        try:
+            return len(v)
+        except (AttributeError, TypeError, ValueError):
+            raise ValueError('Unable to convert %s to int' % (v,))
 
 
 def split_shuffle_and_join(entries: tp.List[T],
-                           whether_to_shuffle: tp.Callable[[T], bool] = lambda x: True,
+                           whether_to_shuffle: Predicate[T] = lambda x: True,
                            not_shuffled_to_front: bool = True) -> tp.List[T]:
     """
     Split elements in entries into two groups - one group, called True, is the one for which
@@ -116,7 +160,7 @@ def split_shuffle_and_join(entries: tp.List[T],
 
 def stringify(obj: tp.Union[tp.Any], stringifier: tp.Callable[[tp.Any], str] = str,
               recursively: bool = False,
-              str_none: bool = False) -> tp.Dict[str, str]:
+              str_none: bool = False) -> tp.Union[tp.List[str], tp.Dict[str, str], str]:
     """
     Stringify all object:
 
@@ -124,16 +168,28 @@ def stringify(obj: tp.Union[tp.Any], stringifier: tp.Callable[[tp.Any], str] = s
         if a list, put every item through stringify
         else just call stringify on it.
 
-        Note that if you use recursively, then dicts and lists are allowed to be valid elements of the returned
-        representation!
+    Note that if you use recursively, then dicts and lists are allowed to be valid elements of the
+    returned representation!
+
+    Note that enums will be converted to their labels. eg:
+
+    >>> class Enum(enum.Enum):
+    >>>     A = 0
+    >>> assert stringify(Enum.A) == 'A'
 
     :param obj: a list or a dict
     :param stringifier: function that accepts any arguments and returns a string representation
-    :param recursively: whether to recursively stringify elements, ie. stringify will be called on all the children
-    :param str_none: whether to return None if given a None. If True, "None" will be returned instead
+    :param recursively: whether to recursively stringify elements, ie. stringify will be called on
+        all the children
+    :param str_none: whether to return None if given a None. If True, "None" will be returned
+        instead
     :return: stringified object
     """
-    if isinstance(obj, collections.abc.Mapping):
+    if isinstance(obj, str):
+        return obj
+    elif isinstance(obj, enum.Enum):
+        return obj.name
+    elif isinstance(obj, collections.abc.Mapping):
         make_str = (lambda obj2: stringify(obj2, stringifier, True, str_none)) if recursively else \
             stringifier
         return {make_str(k): make_str(v) for k, v in obj.items()}
