@@ -3,7 +3,6 @@ import time
 import typing as tp
 
 from satella.coding.decorators import for_argument
-
 from ..data import MetricData, MetricDataCollection
 
 
@@ -83,8 +82,9 @@ class Metric:
             # type: tp.Optional[float]
 
         assert not (
-                self.name == '' and self.level == MetricLevel.INHERIT), 'Unable to set INHERIT for root metric!'
-        self.children = []      # type: tp.List[Metric]
+                self.name == '' and self.level == MetricLevel.INHERIT), \
+            'Unable to set INHERIT for root metric!'
+        self.children = []  # type: tp.List[Metric]
 
     def get_timestamp(self) -> tp.Optional[float]:
         """Return this timestamp, or None if no timestamp support is enabled"""
@@ -104,7 +104,8 @@ class Metric:
     @for_argument(None, MetricLevel)
     def level(self, value: MetricLevel) -> None:
         assert not (
-                    value == MetricLevel.INHERIT and self.name == ''), 'Cannot set INHERIT for the root metric!'
+                value == MetricLevel.INHERIT and self.name == ''), \
+            'Cannot set INHERIT for the root metric!'
         self._level = value
 
     def append_child(self, metric: 'Metric'):
@@ -116,7 +117,8 @@ class Metric:
     def to_metric_data(self) -> MetricDataCollection:
         output = MetricDataCollection()
         for child in self.children:
-            output += child.to_metric_data()
+            if child.level <= self.level:
+                output += child.to_metric_data()
         output.prefix_with(self.name)
 
         if self.enable_timestamp:
@@ -218,7 +220,8 @@ class EmbeddedSubmetrics(LeafMetric):
         if self.embedded_submetrics_enabled:
             v = MetricDataCollection()
             for child in self.children:
-                v = v + child.to_metric_data()
+                if child.level <= self.level:
+                    v = v + child.to_metric_data()
             return v
         else:
             return super().to_metric_data()
@@ -238,3 +241,8 @@ class EmbeddedSubmetrics(LeafMetric):
 
         return self.__class__(self.name, self, MetricLevel.INHERIT, *self.args, labels=labels,
                               **self.kwargs)
+
+
+from .registry import register_metric
+
+register_metric(Metric)
