@@ -1,12 +1,10 @@
 from .daemon import daemonize
+from .misc import suicide, is_running_as_root, whereis
 from .pidlock import PIDFileLock
 from .signals import hang_until_sig
-from .misc import suicide, is_running_as_root
 
 __all__ = [
-    'daemonize',
+    'daemonize', 'whereis',
     'PIDFileLock', 'hang_until_sig',
     'suicide', 'is_running_as_root'
 ]
-
-
