@@ -1,19 +1,27 @@
 import subprocess
-import threading
 import typing as tp
 
 from satella.coding.recast_exceptions import silence_excs
-
+from .coding.concurrent import call_in_separate_thread
 from .exceptions import ProcessFailed
 
-__all__ = ['call_and_return_stdout']
+__all__ = ['call_and_return_stdout', 'read_nowait']
+
+from .time import parse_time_string
 
 
-@silence_excs((IOError, OSError))
-def _read_nowait(process: subprocess.Popen, output_list: tp.List[str]) -> None:
+@call_in_separate_thread(daemon=True)
+@silence_excs(IOError, OSError)
+def read_nowait(process: subprocess.Popen, output_list: tp.List[str]):
     """
-    To be launched as a daemon thread. This reads stdout and appends it's entries to a list.
-    This should finish as soon as the process exits or closes it's stdout.
+    This spawns a thread to read given process' stdout and append it to a list, in
+    order to prevent buffer filling up completely.
+
+    To retrieve entire stdout after process finishes do
+
+    >>> ''.join(list)
+
+    This thread will terminate automatically after the process closes it's stdout or finishes.
     """
     while True:
         with silence_excs(subprocess.TimeoutExpired):
@@ -26,7 +34,7 @@ def _read_nowait(process: subprocess.Popen, output_list: tp.List[str]) -> None:
 
 
 def call_and_return_stdout(args: tp.Union[str, tp.List[str]],
-                           timeout: tp.Optional[int] = None,
+                           timeout: tp.Optional[tp.Union[str, int]] = None,
                            encoding: tp.Optional[str] = None,
                            expected_return_code: tp.Optional[int] = None,
                            **kwargs) -> tp.Union[bytes, str]:
@@ -40,7 +48,8 @@ def call_and_return_stdout(args: tp.Union[str, tp.List[str]],
 
     :param args: arguments to run the program with. Can be either a string or a list of strings.
     :param timeout: amount of seconds to wait for the process result. If process does not complete
-        within this time, it will be sent a SIGKILL
+        within this time, it will be sent a SIGKILL. Can be also a time string. If left at default,
+        ie. None, timeout won't be considered at all.
     :param encoding: encoding with which to decode stdout. If none is passed, it will be returned as
         a bytes object
     :param expected_return_code: an expected return code of this process. 0 is the default. If
@@ -54,20 +63,19 @@ def call_and_return_stdout(args: tp.Union[str, tp.List[str]],
     stdout_list = []
 
     proc = subprocess.Popen(args, **kwargs)
-    reader_thread = threading.Thread(name='stdout reader',
-                                     target=_read_nowait,
-                                     args=(proc, stdout_list),
-                                     daemon=True)
-    reader_thread.start()
+    fut = read_nowait(proc, stdout_list)
+
+    if timeout is not None:
+        timeout = parse_time_string(timeout)
 
     try:
         proc.wait(timeout=timeout)
     except subprocess.TimeoutExpired:
         proc.kill()
         proc.wait()
-        raise TimeoutError('Process did not complete within %s seconds' % (timeout, ))
+        raise TimeoutError('Process did not complete within %s seconds' % (timeout,))
     finally:
-        reader_thread.join()
+        fut.result()
 
     if encoding is None:
         result = b''.join(stdout_list)
